		/* A simple server in the internet domain using TCP
   The port number is passed as an argument */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <libgen.h>
#include <signal.h>
#include <limits.h>
#include "LinuxDARwIn.h"
#include "mjpg_streamer.h"

#define INI_FILE_PATH       "../../../Data/config.ini"

#ifdef MX28_1024
#define MOTION_FILE_PATH    "../../../Data/motion_1024.bin"
#else
#define MOTION_FILE_PATH    "../../../Data/motion_4096.bin"
#endif

#define INI_FILE_PATH       "../../../Data/config.ini"
#define SCRIPT_FILE_PATH    "script.asc"

#define U2D_DEV_NAME0       "/dev/ttyUSB0"
#define U2D_DEV_NAME1       "/dev/ttyUSB1"

using namespace Robot;

void error(const char *msg)
{
    perror(msg);
    exit(1);
}

int main(int argc, char *argv[])
{
	int pitchpos, rollpos,r_pos,p_pos, stand, sit;
	stand = 0;
	sit = 1;
	printf( "\n===== Master/Slave: Server for DARwIn =====\n\n");
	//////////////////// Framework Initialize ////////////////////////////
	LinuxCM730 linux_cm730("/dev/ttyUSB0");
	CM730 cm730(&linux_cm730);
	if(cm730.Connect() == false)
	{
		printf("Fail to connect CM-730!\n");
		return 0;
	}
	/////////////////////////////////////////////////////////////////////
	minIni* ini = new minIni(INI_FILE_PATH);
    	if(MotionManager::GetInstance()->Initialize(&cm730) == false)
   	{
   	     printf("Initializing Motion Manager failed!\n");
   	     return 0;
	}
  	MotionManager::GetInstance()->LoadINISettings(ini);
	Walking::GetInstance()->LoadINISettings(ini);
	MotionManager::GetInstance()->AddModule((MotionModule*)Walking::GetInstance());
	MotionManager::GetInstance()->AddModule((MotionModule*)Action::GetInstance());
	//MotionManager::GetInstance()->AddModule((MotionModule*)Head::GetInstance());
	LinuxMotionTimer *motion_timer = new LinuxMotionTimer(MotionManager::GetInstance());
	
    	motion_timer->Start();

    int firm_ver = 0;
    if(cm730.ReadByte(JointData::ID_HEAD_PAN, MX28::P_VERSION, &firm_ver, 0)  != CM730::SUCCESS)
    {
        fprintf(stderr, "Can't read firmware version from Dynamixel ID %d!! \n\n", JointData::ID_HEAD_PAN);
        exit(0);
    }

    if(0 < firm_ver && firm_ver < 27)
    {
#ifdef MX28_1024
        Action::GetInstance()->LoadFile(MOTION_FILE_PATH);
#else
        fprintf(stderr, "MX-28's firmware is not support 4096 resolution!! \n");
        fprintf(stderr, "Upgrade MX-28's firmware to version 27(0x1B) or higher.\n\n");
        exit(0);
#endif
    }
    else if(27 <= firm_ver)
    {
#ifdef MX28_1024
        fprintf(stderr, "MX-28's firmware is not support 1024 resolution!! \n");
        fprintf(stderr, "Remove '#define MX28_1024' from 'MX28.h' file and rebuild.\n\n");
        exit(0);
#else
        Action::GetInstance()->LoadFile((char*)MOTION_FILE_PATH);
#endif
    }
    else
        exit(0);

    Action::GetInstance()->m_Joint.SetEnableBody(true, true);
    MotionManager::GetInstance()->SetEnable(true);

    Action::GetInstance()->Start(15);

    while(Action::GetInstance()->IsRunning()) usleep(5*1000);

    int sockfd, newsockfd, portno;
    socklen_t clilen;

    int disp[8] = {0};
    struct sockaddr_in serv_addr, cli_addr;
    int n;
    if (argc < 2) {
        fprintf(stderr,"ERROR, no port provided\n");
        exit(1);
    }

    sockfd = socket(AF_INET, SOCK_STREAM, 0);
    if (sockfd < 0)
        error("ERROR opening socket");
    bzero((char *) &serv_addr, sizeof(serv_addr));
    portno = atoi(argv[1]);
    serv_addr.sin_family = AF_INET;
    serv_addr.sin_addr.s_addr = INADDR_ANY;
    serv_addr.sin_port = htons(portno);
    if (bind(sockfd, (struct sockaddr *) &serv_addr,
             sizeof(serv_addr)) < 0)
        error("ERROR on binding");
    listen(sockfd,5);
    clilen = sizeof(cli_addr);
    newsockfd = accept(sockfd,
                       (struct sockaddr *) &cli_addr,
                       &clilen);
    if (newsockfd < 0)
        error("ERROR on accept");

    //store the initial position of dynamixel
    cm730.ReadWord(JointData::ID_L_SHOULDER_PITCH, MX28::P_PRESENT_POSITION_L, &pitchpos, 0);
    cm730.ReadWord(JointData::ID_L_SHOULDER_ROLL, MX28::P_PRESENT_POSITION_L, &rollpos, 0);

    while (1) {

        //read TCP data from client and store it in disp[] array
        n = read(newsockfd,disp,sizeof(disp));
        if (n < 0) error("ERROR reading from socket");

        //new position for shoulder roll and pitch dynamixel is the sum of initial position and angle inputs from Wii remote
        r_pos = rollpos+11.378*disp[0];
        p_pos = pitchpos+11.378*disp[1];

        if (disp[7] ==1)
        {

            Action::GetInstance()->m_Joint.SetEnableBody(false, false);
            MotionManager::GetInstance()->SetEnable(false);

            //dynamixel limits
            if (rollpos>=2460 || rollpos<=1200)
                printf("Limit has been reached");
            else
                cm730.WriteWord(JointData::ID_L_SHOULDER_ROLL, MX28::P_GOAL_POSITION_L, r_pos, 0); //move shoulder roll

            //dynamixel limits
            if (pitchpos<0 || pitchpos>3038)
                printf("Limit has been reached");
            else
                cm730.WriteWord(JointData::ID_L_SHOULDER_PITCH, MX28::P_GOAL_POSITION_L, p_pos, 0); //move shoulder pitch

        }

        //disp[6] stores states of button A on Wii remote
        //integer stand and sit stores the state of DARwIn-OP
        if (disp[6] == 1)
        {
            if(stand == 0)
            {
                if(Action::GetInstance()->IsRunning() == 0)
                {
                    Action::GetInstance()->m_Joint.SetEnableBody(true, true);
                    MotionManager::GetInstance()->SetEnable(true);
                    Action::GetInstance()->Start(9);
                    stand = 1;
                    sit = 0;
                }
            }
        }
        else if (disp[6] == -1)
        {
            if (sit == 0)
            {
                if(Action::GetInstance()->IsRunning() == 0)
                {
                    Action::GetInstance()->m_Joint.SetEnableBody(true, true);
                    MotionManager::GetInstance()->SetEnable(true);
                    Action::GetInstance()->Start(15);
                    sit = 1;
                    stand = 0;
                }
            }
        }

        n = write(newsockfd,"0",1);
        if (n < 0) error("ERROR writing to socket");
    }

    close(newsockfd);
    close(sockfd);
    return 0;
}
